/* Microsoft C and Turbo C Interrupt Service Sample Program
   Program Name: INTERUPT.C

    To create an executable program, INTERUPT.EXE, (medium memory model),
    with the command line compiler and linker, use the following:

       Microsoft C:    cl /AM interupt.c mcm_mm.obj
       Borland C:      tcc -mm interupt.c tcm_mm.obj

    .  mcm_mm.obj is on your MotionMaster 2000 diskette in directory /MC51
    .  tcm_mm.obj is on your MotionMaster 2000 diskette in directory /TC20
*/

#include <stdio.h>
#include <string.h>
#include <dos.h>

/* Definition for the 8259 Programmable Interrupt Controller (PIC) */
#define P8259_0  0x20   /* Address of interrupt status register */
#define P8259_1  0x21   /* Address of interrupt mask register */
#define irq_done 0x20   /* Written to PIC at end of interrupt service */

/* MotionMaster 2000 configuration */
#define mm2000_irq  3   /* Assume MM2000 J18 configured for IRQ3 */
#define mm_add  0x340   /* Assume MM2000 J16 configured for address 340h */

void interrupt far new_vector (void);
void (interrupt far *old_vector) (void);
void mm_int_setup(void);
void clean_up (void);

int mm_irq_flag=0, intmask;
int int_disable_mask=0x00, int_number=0, int_enable_mask=0xff ;
int k=0;

char  response[80], *cmd_ptr, cmd_copy[80];
char  *cmd_array[] =
{
"1FI11\r",             /* generate interrupt on axis 1 motion complete or error */
"1VA1000;AC50000\r",   /* set axis 1 velocity and acceleration */
"1DH;PR2000\r",        /* clear position counter and move axis 1 */
NULL
};

main ()
{
int cmd_num=0, mm_position;

     mm_int_setup();   /* unmask 8259 Programmable Interrupt Controller */
     old_vector = _dos_getvect (mm2000_irq + 8); /* copy originl vector */
     _dos_setvect (mm2000_irq + 8, new_vector);  /* install user vector */

     while (cmd_array[cmd_num])     /* transmit array */
        {
        strcpy(cmd_copy,cmd_array[cmd_num]);
        mmsend(cmd_copy,mm_add);    /* pass command & board addr. */
        ++cmd_num;                  /* point to next command string */
        }

     printf("\n\rProcessing other tasks during move: Incrementing a variable !!!\n");

     while (mm_irq_flag == 0){      /* wait for MotionMaster interrupt */
          ++k;
          printf("\r%i",k);
          }

      mmsend("1TP\r",mm_add);   /* pass command & board addr.*/
      mmread(response,mm_add);  /* get response from MotionMaster */
      printf("\r\n\nMotor Position After First Move = %s\r",response);

      printf("\n\nAnother move initiated ...\n");
      mmsend("1FI00\r",mm_add);
      mmsend("1PR-2000\r",mm_add);

clean_up ();         /* restore original vector and mask irq */
}


void interrupt far new_vector (void)
{
     mm_irq_flag=1;            /* set MotionMaster interrupt flag and...*/
     outp(P8259_0, irq_done);  /* tell PIC irq service is done */
}


void clean_up (void)
{
     _dos_setvect (mm2000_irq + 8,old_vector);  /* restore original vector */

     intmask = inp(P8259_1) | int_disable_mask; /* mask interrupt */
     outp(P8259_1, intmask);
}

void mm_int_setup(void)
{

     if (mm2000_irq == 3) {           /* setup for IRQ3 */
        int_enable_mask = 0xf7;
        int_disable_mask = 0x08;
        int_number = 11;
        }

     if (mm2000_irq == 5) {            /* setup for IRQ5 */
        int_enable_mask = 0xdf;
        int_disable_mask = 0x20;
        int_number = 13;
        }

     if (mm2000_irq == 7) {            /* setup for IRQ7 */
        int_enable_mask = 0x7f;
        int_disable_mask = 0x80;
        int_number = 15;
        }

intmask = inp(P8259_1) & int_enable_mask; /* AND 8259's irq mask register..*/

outp(P8259_1, intmask);                   /*...with int_enable_mask byte */
}
